------------------------------------------------------
--! @file mux_16to1_tb.vhd
--! @brief Testbench for 16 to 1 multiplexer
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2020-04-01
-------------------------------------------------------

--  A testbench has no ports.
entity mux_16to1_tb is
end entity mux_16to1_tb;

architecture testbench of mux_16to1_tb is

  --  Declaration of the component to be tested.  
  component mux_16to1
  port (
    SEL : in bit_vector (3 downto 0);
    I : in bit_vector (15 downto 0);
    Y : out bit
    );
  end component mux_16to1;

  -- Declaration of signals
  signal sel : bit_vector (3 downto 0);
  signal i : bit_vector (15 downto 0);
  signal y : bit;

begin
  -- Component instantiation
  -- DUT = Device Under Test 
  DUT: entity work.mux_16to1(with_select) port map (
    SEL => sel,
    I => i,
    Y => y
    );

  --  This process does the real job.
  stimulus_process: process is
    type pattern_type is record
      --  The inputs of the circuit.
      sel : bit_vector (3 downto 0);
      i : bit_vector (15 downto 0);
      
      --  The expected outputs of the circuit.
      y : bit;
    end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (
        ("0000","0000000000000011",'1'),        -- I0
        ("0000","0000000000000010",'0'),
        ("0001","0000000000000011",'1'),        -- I1
        ("0001","0000000000000001",'0'),
        ("0010","0000000000000101",'1'),        -- I2
        ("0010","0000000000000001",'0'),
        ("0011","0000000000001001",'1'),        -- I3
        ("0011","0000000000000001",'0'),
        ("0100","0000000000010001",'1'),        -- I4
        ("0100","0000000000000001",'0'),
        ("0101","0000000000100001",'1'),        -- I5
        ("0101","0000000000000001",'0'),
        ("0110","0000000001000001",'1'),        -- I6
        ("0110","0000000000000001",'0'),
        ("0111","0000000010000001",'1'),        -- I7
        ("0111","0000000000000001",'0'),
        ("1000","0000000100000001",'1'),        -- I8
        ("1000","0000000000000001",'0'),
        ("1001","0000001000000001",'1'),        -- I9
        ("1001","0000000000000001",'0'),
        ("1010","0000010000000001",'1'),        -- I10
        ("1010","0000000000000001",'0'),
        ("1011","0000100000000001",'1'),        -- I11
        ("1011","0000000000000001",'0'),
        ("1100","0001000000000001",'1'),        -- I12
        ("1100","0000000000000001",'0'),
        ("1101","0010000000000001",'1'),        -- I13
        ("1101","0000000000000001",'0'),
        ("1110","0100000000000001",'1'),        -- I14
        ("1110","0000000000000001",'0'),
        ("1111","1000000000000001",'1'),        -- I15
        ("1111","0000000000000001",'0'));
    
  begin        
    --  Check each pattern.
    for k in patterns'range loop

      --  Set the inputs.
      sel <= patterns(k).sel;
      i <= patterns(k).i;
      
       --  Wait for the results.
      wait for 1 ns;
      
  --  Check the outputs.

      assert y = patterns(k).y        
        report "bad y" severity error;
    end loop;
    
    assert false report "end of test" severity note;
    
  --  Wait forever; this will finish the simulation.
    wait;
  end process;
  
end architecture testbench;
  

