
-------------------------------------------------------
--! @file mux_2to1_tb.vhd
--! @brief Testbench for 2 to 1 multiplexer
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2020-04-01
-------------------------------------------------------

--  A testbench has no ports.
entity mux_2to1_tb is
end entity mux_2to1_tb;

architecture testbench of mux_2to1_tb is

  --  Declaration of the component to be tested.  
  component mux_2to1
  port (
    SEL : in  bit;
    I   : in  bit_vector (1 downto 0);
    Y   : out bit
    );
  end component mux_2to1;

  -- Declaration of signals
  signal sel : bit;
  signal i   : bit_vector (1 downto 0);
  signal y   : bit;

begin
  -- Component instantiation
  -- DUT = Device Under Test 
  DUT: entity work.mux_2to1(with_select) port map (
    SEL => sel,
    I => i,
    Y => y
    );

  --  This process does the real job.
  stimulus_process: process is
    type pattern_type is record
      --  The inputs of the circuit.
      sel : bit;
      i   : bit_vector (1 downto 0);
      
      --  The expected outputs of the circuit.
      y : bit;
    end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (
        ('0',"10",'0'),
        ('0',"01",'1'),
        ('1',"01",'0'),
        ('0',"10",'1'));

  begin        
    --  Check each pattern.
    for k in patterns'range loop

      --  Set the inputs.
      sel <= patterns(k).sel;
      i <= patterns(k).i;
      
       --  Wait for the results.
      wait for 1 ns;
      
  --  Check the outputs.

      assert y = patterns(k).y        
        report "bad y" severity error;
    end loop;
    
    assert false report "end of test" severity note;
    
  --  Wait forever; this will finish the simulation.
    wait;
  end process;


  
end architecture testbench;
  

