-------------------------------------------------------
--! @file sub4_tb.vhd
--! @brief Testbench for 4-bit subtractor
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2018-10-07
-------------------------------------------------------

entity sub4_tb is
end entity sub4_tb;

architecture arch of sub4_tb is

-- Component Declaration
component sub4 is
  port (
    BIN : in bit;
    X : in bit_vector (3 downto 0);
    Y : in bit_vector (3 downto 0);
    DIFF : out bit_vector (3 downto 0);
    BOUT : out bit
    );
end component sub4;

-- Signals
signal bin : bit;
signal x : bit_vector (3 downto 0);
signal y : bit_vector (3 downto 0);
signal diff : bit_vector (3 downto 0);
signal bout : bit;

begin
  -- Component Instantiation
  USUB4 : component sub4 port map (bin,x,y,diff,bout);
    
  -- Stimulus process
  stimulus_process : process is
    type pattern_type is record
     -- The inputs of the circuit
     bin : bit;
     x : bit_vector (3 downto 0);
     y : bit_vector (3 downto 0); 
     -- The expected outputs of the circuit
     diff : bit_vector (3 downto 0);
     bout : bit;
     end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (('0',"0001","0010","1111",'0'),
       ('1',"0001","0010","0000",'1'),
       ('0',"1111","1110","0001",'1'),
       ('0',"1111","0001","1110",'1'),
       ('0',"0000","0001","1111",'0'),
       ('0',"1010","0101","0101",'1'),
       ('1',"1010","0101","0110",'1'),
       ('0',"0111","0001","0110",'1'),
       ('1',"0111","0001","0111",'1'),
       ('0',"1000","1000","0000",'1'),
       ('1',"1000","1000","0001",'1'));

    begin

    wait for 1 ns;
      
    --  Check each pattern.
    for k in patterns'range loop
      --  Set the inputs.
      bin <= patterns(k).bin;
      x <= patterns(k).x;
      y <= patterns(k).y;
  
    --  Wait for the results.
    wait for 3 ns;
    --  Check the outputs.
    assert diff = patterns(k).diff
      report "bad check" severity error;
    assert bout = patterns(k).bout
      report "bad check" severity error;
    end loop;
    assert false report "end of test" severity note;
    --  Wait forever; this will finish the simulation.
      wait;
 end process;

end architecture arch;
